// SCardConnection.h: interface for the CSmartCardConnection class.
//
//////////////////////////////////////////////////////////////////////

#pragma warning( disable : 4290)

#if !defined(AFX_SCARDCONNECTION_H__E82E75E1_713F_4831_B8A1_ED8BDD7F5F32__INCLUDED_)
#define AFX_SCARDCONNECTION_H__E82E75E1_713F_4831_B8A1_ED8BDD7F5F32__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "USCAPI.h"
#include <winscard.h>

#define MAX_READER_NAME  256

#define ENCODE_SW1SW2(sw1, sw2) (sw1*256 + sw2)
#define SW1(val) (HIBYTE(val))
#define SW2(val) (LOBYTE(val))

#define PCSC_OK(val) (HIWORD(val) == 0xFFFF)
#define APDU_OK(val) (((SW1(val) == 0x90) || (SW1(val) == 0x91) || (SW1(val) == 0x9F) || (SW1(val) == 0x61)))

namespace USCAPI
{

///Defines the Command APDU structure
class USCAPI_API CCommandAPDU
{
public:
	///CLAss Byte
	BYTE  cla;		
	///INStruction byte 
	BYTE  ins;		
	///Paramenter 1 byte
	BYTE  p1;		
	///Paramente 2 byte
	BYTE  p2;		

	union
	{
		///Length of data fiels
		short  lc;		
		///In T=0 protocol, Length of data fiels or Length of expected bytes
		short  p3;	    
	};


	/// Data field
	BYTE pbtData[1000];
	///Length of Expected data
	short  le;		

	///Default constructor
	CCommandAPDU();
	///Constructor for a read only APDU
	CCommandAPDU(BYTE _cla, BYTE _ins, BYTE _p1, BYTE _p2, short _p3_le);
	///Constructor for a read/write APDU
	CCommandAPDU(BYTE _cla, BYTE _ins, BYTE _p1, BYTE _p2, short _p3_lc, BYTE* _pbtData);	
	///Constructor for a T=1 read/write APDU
	CCommandAPDU(BYTE _cla, BYTE _ins, BYTE _p1, BYTE _p2, short _lc, BYTE* _pbtData, short _le);
	///Constructor for raw protocol APDU
	CCommandAPDU(BYTE* _pbtData, short len);

	/// gets the APDU buffer
	long getAPDU(BYTE* pbtAPDU, short* pLen) const;

	/// Set the DATA fiels
	void setData(BYTE* pbtData, short len);
};

///Defines the Response APDU structure
class USCAPI_API CResponseAPDU
{
public:
	///Status Word 1
	BYTE sw1;		
	/// Status Word 2
	BYTE sw2;		
	/// Data Field
	BYTE pbData[256];	
	/// Data Length
	short dataLen;		
	
	///Constructor
	CResponseAPDU(BYTE sw1, BYTE sw2, BYTE* data, short datalen);
	///Constructor
	CResponseAPDU(){};

	///Check for success
	BOOL success();

	///SW1SW2 APDU response
	short SW1SW2();
};

/// ATR data structure
typedef struct USCAPI_API ATR
{
	ATR()
	{
		nLen = 40;
	}

	BYTE btATR[40];
	unsigned long nLen;
} _ATR;

/// Implements a Smart Card connection
class USCAPI_API CSmartCardConnection  
{
	friend class CSmartCardManager;

public:		

	virtual ~CSmartCardConnection();

	/// Begins a transaction with the smart card
	long beginTransaction();
	
	/// Ends a transaction with the smart card
	long endTransaction();

	/// Gets the name of the connected reader
	LPCSTR getReaderName();
	
	/// Sends an APDU
	long send(IN const CCommandAPDU& apdu, CResponseAPDU* pResponse);
	/// Sends an APDU
	long send(IN const BYTE* pbtApdu, int nApduLen, CResponseAPDU* pResponse);

	/// Gets the smart card's ATR
	const ATR& getATR() const;

	/// Gets the smart card's protocol
	DWORD getProtocol();

	/// Checks for card presence
	bool isSmartCardPresent();

protected:

	/// Opens a connection
	static CSmartCardConnection* openConnection(SCARDCONTEXT hContext, LPCSTR lpszReaderName, DWORD dwSharedMode, DWORD dwProtocol);	
	/// Cloese a connection
	static long closeConnection(IN CSmartCardConnection* connection, bool bReset = false);	

private:
	CSmartCardConnection(SCARDCONTEXT hContext, SCARDHANDLE hCard, LPCSTR lpszReaderName, DWORD dwSharedMode, DWORD dwProtocol, ATR& atr);

	char			m_szReaderName[MAX_READER_NAME];
	SCARDHANDLE		m_hCard;
	SCARDCONTEXT	m_hContext;
	DWORD			m_dwProtocol;
	DWORD			m_dwSharedMode;
	ATR				m_atr;

	CSmartCardManager* m_pscMan;   
};

}
#endif // !defined(AFX_SCARDCONNECTION_H__E82E75E1_713F_4831_B8A1_ED8BDD7F5F32__INCLUDED_)
