// SCardManager.h: interface for the CSmartCardManager class.
//
//////////////////////////////////////////////////////////////////////
#pragma warning( disable : 4290)

#if !defined(AFX_SCARDMANAGER_H__DED857A2_566E_49A1_AFF9_393682240D18__INCLUDED_)
#define AFX_SCARDMANAGER_H__DED857A2_566E_49A1_AFF9_393682240D18__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <winscard.h>
#include "uscapi.h"
#include "SmartCardConnection.h"
#include "SmartCardReaderVector.h"

#define SC_PRESENT(dwState) \
	((dwState & SCARD_STATE_PRESENT) == SCARD_STATE_PRESENT)

namespace USCAPI
{

/// Implements the smart card manager for managing all stuffs related to the smart card.
/// This is the main class of this API
class USCAPI_API CSmartCardManager  
{
public:

	/// Constructor
	CSmartCardManager(const SCARDCONTEXT hContext);
	/// Constructor
	CSmartCardManager();
	~CSmartCardManager();

	/// Refreshes Smart Card Context
	long refreshContext();

	/// Opens a new Smart Card Connection
	CSmartCardConnection* connect(const char* szReader = NULL, IN DWORD dwShareMode = SCARD_SHARE_SHARED, IN DWORD dwProtocol = SCARD_PROTOCOL_T1 | SCARD_PROTOCOL_T0);

	/// Disconnects
	long disconnect(CSmartCardConnection* pCardConnection, bool bReset = false);
	
	/// Gets the list of plugged readers
	long getReaderList(CSmartCardReaderVector& readerVector, bool bPresent) const;

	/// Gets the reader state
	long getReaderState(const char* szReader, DWORD* pdwState) const;

	/// Checks for smart card presence
	bool isSmartCardInserted(const char* szReader) const;

	/// Waits for smart card insertion 
	long waitForSmartCardInserted(const char* szReader);
	/// Waits for smart card insertion 
	long waitForSmartCardInserted(const char* szReader, int timeout);
	/// Waits for smart card removal
	long waitForSmartCardRemoved(const char* szReader);
	/// Waits for smart card removal
	long waitForSmartCardRemoved(const char* szReader, int timeout);

	/// Cancels all waiting operations
	long cancelWaiting();

private:
	SCARDCONTEXT m_hContext;
};

}
#endif // !defined(AFX_SCARDMANAGER_H__DED857A2_566E_49A1_AFF9_393682240D18__INCLUDED_)
