// SCardReaderListener.h: interface for the CSmartCardReaderListener class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCARDREADERLISTENER_H__F276A1D1_139A_45E4_B9FA_425EB1A22C88__INCLUDED_)
#define AFX_SCARDREADERLISTENER_H__F276A1D1_139A_45E4_B9FA_425EB1A22C88__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SmartCardManager.h"
#include "uscapi.h"
#include <UUCHashtable.hpp>

#define CARD_EVT_REMOVED			  1
#define CARD_EVT_INSERTED			  2
#define CARD_EVT_UNABLETOOPENCONTEXT  3
#define CARD_EVT_CANCELLED			  4
#define CARD_EVT_READER_UNAVAILABLE	  5
#define CARD_EVT_NO_READERS_AVAILABLE 6

typedef UUCHashtable<int, SCARDCONTEXT> MONITORTABLE;

namespace USCAPI
{

/// Implements the reader listener to listen to smart card reader events such as insertion or removal
class USCAPI_API CSmartCardReaderListener  
{
public:	

	/// Constructor
	CSmartCardReaderListener();
	virtual ~CSmartCardReaderListener();

	/// Notification for no readers available
	virtual void noReadersAvailable();
	/// Notification for reader unavailable
	virtual void readerUnavailable(const char* szReader);
	/// Notification for operation cancelled
	virtual void cancelled(const char* szReader);
	/// Notification for unable to open context
	virtual void unableToOpenContext(const char* szReader);
	/// Notification for smart card removed
	virtual void smartCardRemoved(const char* szReader);
	/// Notification for smart card inserted
	virtual void smartCardInserted(const char* szReader, const ATR& atr);		

	/// Enables/Disables the monitoring thread
	bool enable(bool bEnable);
	/// Enables/Disables the monitoring thread for the given reader
	bool enable(bool bEnable, const char* szReader);
	/// Check if the monitoring is enabled
	bool isEnabled() const;

	/// Check if the monitoring is enabled for the given reader	
	bool isEnabled(const char* szReader) const;

private:
	bool m_bEnabled;
	CSmartCardManager m_scMgr;
	HANDLE m_hCloseEvent;
	MONITORTABLE m_monitorTbl;	

	friend void readerMonitoring(void*);
};

}
#endif // !defined(AFX_SCARDREADERLISTENER_H__F276A1D1_139A_45E4_B9FA_425EB1A22C88__INCLUDED_)
