import java.io.ByteArrayInputStream;
import java.security.Principal;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

import com.cryptware.crypto.cert.CertViewer;
import com.cryptware.jcnsapi.CNS;
import com.cryptware.jscapi.PCSCException;
import com.cryptware.jscapi.Reader;
import com.cryptware.jscapi.SmartCard;
import com.cryptware.jscapi.SmartCardException;
import com.cryptware.jscapi.SmartCardManager;


public class TestCNSAPI {

	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
		try
		{
			SmartCardManager scman = new SmartCardManager();
	
	        List<Reader> readerList = scman.getPluggedReaders();
	
	        if(readerList.size() == 0)
	        {
	        	System.out.println("No readers found");
	        	return;		        	
	        }
	        
	        Reader reader = readerList.get(0);
	
	        System.out.println(reader.getName());       
	        System.out.println(reader.isSmartCardPresent());
	
	        reader.WaitForSmartCardInserted();
	      
	        SmartCard card = reader.connect();
	        
	        CNS cns = new CNS(card);
	
	        // sends the command APDU and gets the Response APDU
        	try 
        	{
				byte[] cardId = cns.ReadIDCarta();	
				System.out.println("ID Carta: " + new String(cardId));
        	}
	        catch (SmartCardException e1) 
	        {
				System.out.println(e1.getMessage());
				e1.printStackTrace();
			}
//	        
	        // sends the command APDU and gets the Response APDU
	        try 
	        {
				byte[] datiPossessore = cns.ReadDatiPossessore();	
				System.out.println("datiPossessore: " + new String(datiPossessore));
	        }
	        catch (SmartCardException e1) 
	        {
				System.out.println(e1.getMessage());
				e1.printStackTrace();
			}
	
	     // sends the command APDU and gets the Response APDU
	        try
	        {
				byte[] datipersonali = cns.ReadDatiPersonaliRaw();	
				System.out.println("datipersonali: " + new String(datipersonali));
	        }
	        catch (SmartCardException e1) 
	        {
				System.out.println(e1.getMessage());
				e1.printStackTrace();
			}
//	
	        try
	        {
				String[] datipersonali = cns.ReadDatiPersonali();	
				System.out.println("datipersonali: " +  datipersonali.toString());
				System.out.println("Nome, Cognome: " +  datipersonali[CNS.Nome] + " " + datipersonali[CNS.Cognome]);
				
	        }
	        catch (SmartCardException e1) 
	        {
				System.out.println(e1.getMessage());
				e1.printStackTrace();
			}	        
			
			try 
	        {
				X509Certificate certificate = cns.ReadCertificatoX509();
														
				System.out.println("serialnubmer " + certificate.getSerialNumber());
	        }
			catch (CertificateException e) 
			{
				e.printStackTrace();
			}
	        catch (SmartCardException e1) 
	        {
				System.out.println(e1.getMessage());
				e1.printStackTrace();
			}
	        
	     // sends the command APDU and gets the Response APDU
	        try 
	        {
				byte[] cert1 = cns.ReadCertificato();
								
				System.out.println("Cert: " + new String(cert1));
	        }
	        catch (SmartCardException e1) 
	        {
				System.out.println(e1.getMessage());
				e1.printStackTrace();
			}
		}
		catch(PCSCException ex)
		{
			ex.printStackTrace();
		}
	}

}
